/*
** main.c
**
** Nokia 3310 Example Program
** Target: ATMEGA :: AVR-GCC
**
** Written by Tony Myatt - 2007
** Quantum Torque - www.quantumtorque.com
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define BLOCK_SIZE	1
#define INIT_SIZE	3
#define MAX_SIZE	20

#define SEED 36

// coord struct
typedef struct {
	unsigned int x;
	unsigned int y;
} coord;

unsigned int direction;

// coords for the snake
coord snake[MAX_SIZE];

unsigned int head_pos;

unsigned int tail_pos;

unsigned int snake_size;

int start = 0;
coord food;

// set the direction according to the pressed button
void set_direction(void) {
	
	// UP
	if ((PIND & (1 << PD0)) == 0 )
		direction = ARROW_UP;
	// DOWN
	else if((PIND & (1 << PD1)) == 0 )
		direction = ARROW_DOWN;
	// LEFT
	else if((PIND & (1 << PD4)) == 0)
		direction = ARROW_LEFT;
	// RIGHT
	else if((PIND & (1 << PD5)) == 0)
		direction = ARROW_RIGHT;
}

// generate random coordinates for food
coord generate_coord(void) {
	coord c;

	c.x = rand() % 14;
	c.y = rand() & 6;
	return c;
}

// init the buttons for moving around
void init_buttons(void) {
	// UP
	DDRD &= ~(1 << PD0);
	PORTD |= (1 << PD0);

	// DOWN
	DDRD &= ~(1 << PD1);
	PORTD |= (1 << PD1);

	// LEFT
	DDRD &= ~(1 << PD4);
	PORTD |= (1 << PD4);

	// RIGHT
	DDRD &= ~(1 << PD5);
	PORTD |= (1 << PD5);
}

// initial setup
void start_game(void) {
	unsigned int i;

	snake_size = INIT_SIZE;

	// calculate the coordinates for snake body
	for(i = 0; i < INIT_SIZE; i++) {
		snake[i].x = (LCD_X_RES - INIT_SIZE) / 2 + i * BLOCK_SIZE;
		snake[i].y = LCD_Y_RES / 2;
	}
	head_pos = INIT_SIZE / 2;
	tail_pos = 0;

	srand(SEED);
	// generate food coordinates
	food = generate_coord();
}

// update the snakes size and coordinates
void update() {
	unsigned int prev;

	prev = head_pos;
	head_pos = (head_pos + 1) % MAX_SIZE;

	snake[head_pos] = snake[prev];

	// when the snake ate a dot 
	if(abs(snake[head_pos].x - food.x) < BLOCK_SIZE && abs(snake[head_pos].y - food.y) < BLOCK_SIZE) {
		food = generate_coord();
		snake_size++;
	} else
		tail_pos = (tail_pos + 1) % MAX_SIZE;

	if(direction == ARROW_UP) {
		snake[head_pos].y = (snake[head_pos].y + BLOCK_SIZE) % 7;
		_delay_ms(200);
	}
	else if(direction == ARROW_DOWN) {
		snake[head_pos].y = (snake[head_pos].y - BLOCK_SIZE) % 7;
		_delay_ms(200);
	}
	else if(direction == ARROW_LEFT) {
		snake[head_pos].x = (snake[head_pos].x - BLOCK_SIZE) % 14;
		_delay_ms(200);
	}
	else {
		snake[head_pos].x = (snake[head_pos].x + BLOCK_SIZE) % 14;
		_delay_ms(200);
	}
}

// draw the current view of the game
void draw_game(void) {
	unsigned int i;
	coord block;

	for(i = 0; i < snake_size; i++) {
		block = snake[(tail_pos + i) % MAX_SIZE];
		lcd_goto_xy(block.x - BLOCK_SIZE / 2, block.y - BLOCK_SIZE / 2);
		lcd_chr(LINE);
	}

	lcd_goto_xy(food.x - BLOCK_SIZE / 2, food.y - BLOCK_SIZE / 2);
	lcd_chr(LINE);

}

/* main -- Program starts here */
int main(void)
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);

	lcd_clear();
	init_buttons();

	direction = STOP;
	
	while(1) {
		set_direction();
		if(direction != STOP)
			break;
		if(start == 0) {
			lcd_str("START GAME");
			start++;
		}
		_delay_ms(100);
	}	

	start_game();

	while(1) {
		set_direction();
		lcd_clear();
		draw_game();
		update();
		_delay_ms(3000);
	}
}
